%Robust Loop Shaping
%Ball and Beam controller
%Strictly proper loop shape D=0

W = tf(5);
s=tf('s');

%Nominal Ball and Beam Model
Kbb= g*5/7;
plant=Kbb/s^2;
pss = ss(plant);
[A1, B1, C1, D1] = ssdata(pss);

%Weighting function
W1 = W;
%W2 = 5*(s+1)/(0.3*s+1);
W2 = k_param*(s+z_param)/(s+p_param);


%loop shape used in design
loopss = pss*W1;
[A B C D] = ssdata(loopss);

loopss2 = pss*W2;
[A2 B2 C2 D2] = ssdata(loopss2);

%Solve two Ricatti equations
x = care(A,B,C'*C);
z = care(A',C',B*B');

x2 = care(A2,B2,C2'*C2);
z2 = care(A2',C2',B2*B2');

%compute gamma 
gamo = sqrt(1+max(eig(x*z)));

gamo2 = sqrt(1+max(eig(x2*z2)));


%Use suboptimal gamma for the design 10 % larger is usual
ten = 1.1;
gam = ten*gamo;
R = eye(size(z*x))-(eye(size(z*x))+z*x)/gam^2;

gam2 = ten*gamo2;
R2 = eye(size(z2*x2))-(eye(size(z2*x2))+z2*x2)/gam2^2;


%Observer
L = B'*x;
K = inv(R)*z*C';
Ao = A-K*C;

L2 = B2'*x2;
K2 = inv(R2)*z2*C2';
A2o = A2-K2*C2;


%Controller (A,B,C,D)
Ac = A-B*L-K*C;
Cc=L;
Dc=0;
Bc=K;

Ac2 = A2-B2*L2-K2*C2;
Cc2=L2;
Dc2=0;
Bc2=K2;


%The robust Controller
comp = ss(Ac,Bc,Cc,Dc);
loop1 = plant*W1*comp;
loop1ss = ss(loop1);

comp2 = ss(Ac2,Bc2,Cc2,Dc2);
loop12 = plant*W2*comp2;
loop1ss2 = ss(loop12);


%Digital controller
W1d = c2d(W1,ts);
[num1d den1d] = tfdata(W1d, 'v')

W1d2 = c2d(W2,ts);
[num2d den2d] = tfdata(W1d2, 'v')


compd=c2d(comp,ts);
[num,den]=tfdata(compd,'v')

compd2=c2d(comp2,ts);
[num2,den2]=tfdata(compd2,'v')

numc1 = conv (num1d,num)
denc1 = conv (den1d,den)

numc = conv (num2d,num2)
denc = conv (den2d,den2)

