% The dynamic model of n-connected bogies. This function calls
% 'bogie_friction_Karnopp.m' for calculating friction forces using karnopp
% model.
% Author: Babak Sedghi, LA-EPFL, babak.sedghi@epfl.ch
% Revision: 1, Date: 15. 5.2004

function [sys,x0,str,ts]=Commande(t,x,u,flag,Param)

switch flag

  %%%%%%%%%%%%%%%%%%
  % Initialization %
  %%%%%%%%%%%%%%%%%%
  case 0         
    [sys,x0,str,ts] = mdlInitializeSizes(Param);

  %%%%%%%%%%%%%%%
  % Derivatives %
  %%%%%%%%%%%%%%%
  case 1 
      
    sys = mdlDerivatives(t,x,u,Param);

  %%%%%%%%%%%%%%%%%%%%%%%%
  % Update and Terminate %
  %%%%%%%%%%%%%%%%%%%%%%%%
  case {2,9}
    sys = []; % do nothing

  %%%%%%%%%%
  % Output %
  %%%%%%%%%%
  case 3
      %keyboard
    sys = mdlOutputs(t,x,u,Param); 
    
  otherwise
    error(['unhandled flag = ',num2str(flag)]);
end

% end limintm

%
%=============================================================================
% mdlInitializeSizes
% Return the sizes, initial conditions, and sample times for the S-function.
%=============================================================================
%
function [sys,x0,str,ts] = mdlInitializeSizes(Param)

sizes = simsizes;
sizes.NumContStates  = 1;
sizes.NumDiscStates  = 0;
sizes.NumOutputs     = 16;
sizes.NumInputs      = 32;
sizes.DirFeedthrough = 32;
sizes.NumSampleTimes = 1;

sys = simsizes(sizes);
str = [];

x0  = [1];
ts  = [0 0];   % sample time: [period, offset]

% end mdlInitializeSizes

%

%=============================================================================
% mdlDerivatives
% Compute derivatives for continuous states.
%=============================================================================
%
function sys = mdlDerivatives(t,x,u,Param)
sys=u(1);
% end mdlDerivatives

%
%=============================================================================
% mdlOutputs
% Return the output vector for the S-function
%=============================================================================
%
function sys = mdlOutputs(t,x,u,Param)

x1C      = u(1);
x1Cd     = u(2); 
x1Cdd    = u(3);
x1Cddd   = u(4);
x1Cdddd  = u(5);

x2C      = u(6);
x2Cd     = u(7);
x2Cdd    = u(8);
x2Cddd   = u(9);
x2Cdddd  = u(10);

x3C      = u(11);
x3Cd     = u(12);
x3Cdd    = u(13);
x3Cddd   = u(14);
x3Cdddd  = u(15);

x03C     = u(16);
x03Cd    = u(17);
x03Cdd   = u(18);
x03Cddd  = u(19);
x03Cdddd = u(20);

L1       = u(21);
L1d      = u(22);
L2       = u(23);
L2d      = u(24);
L3       = u(25);
L3d      = u(26);
L4       = u(27);
L4d      = u(28);

% Jose
KP       = u(29);
KD       = u(30);
g        = u(31);
m        = u(32);
% Jose


mo  = Param.mo;
m1  = Param.m1;
m2  = Param.m2;
m3  = Param.m3; 
m4  = Param.m4;
x11 = Param.x11;
x12 = Param.x12;
x13 = Param.x13;
x21 = Param.x21;
x22 = Param.x22; 
x23 = Param.x23;
x31 = Param.x31;
x32 = Param.x32;
x33 = Param.x33;
x41 = Param.x41;
x42 = Param.x42; 
x43 = Param.x43;



%Calcul de consigne(inverse inverse)
x01C = (g*x1C + x03C*x1Cdd - x1Cdd*x3C + x1C*x3Cdd)*(g + x3Cdd)^(-1);
x02C = (g*x2C + x03C*x2Cdd - x2Cdd*x3C + x2C*x3Cdd)*(g + x3Cdd)^(-1);

x01Cd = -((g*x1C + x03C*x1Cdd - x1Cdd*x3C + x1C*x3Cdd)*x3Cddd*(g + x3Cdd)^(-2)) + ...
      (g*x1Cd + x03Cd*x1Cdd + x03C*x1Cddd - x1Cddd*x3C - x1Cdd*x3Cd + x1Cd*x3Cdd + x1C*x3Cddd)*(g + x3Cdd)^(-1);
x02Cd = -((g*x2C + x03C*x2Cdd - x2Cdd*x3C + x2C*x3Cdd)*x3Cddd*(g + x3Cdd)^(-2)) +... 
      (g*x2Cd + x03Cd*x2Cdd + x03C*x2Cddd - x2Cddd*x3C - x2Cdd*x3Cd + x2Cd*x3Cdd + x2C*x3Cddd)*(g + x3Cdd)^(-1);

x01Cdd = -2*x3Cddd*(g*x1Cd + x03Cd*x1Cdd + x03C*x1Cddd - x1Cddd*x3C - x1Cdd*x3Cd + x1Cd*x3Cdd + x1C*x3Cddd)*(g + x3Cdd)^(-2) -... 
   (g*x1C + x03C*x1Cdd - x1Cdd*x3C + x1C*x3Cdd)*x3Cdddd*(g + x3Cdd)^(-2) + ...
   (g*x1Cdd + x03Cdd*x1Cdd + 2*x03Cd*x1Cddd + x03C*x1Cdddd - x1Cdddd*x3C - 2*x1Cddd*x3Cd + 2*x1Cd*x3Cddd + x1C*x3Cdddd)*...
    (g + x3Cdd)^(-1) + 2*(g*x1C + x03C*x1Cdd - x1Cdd*x3C + x1C*x3Cdd)*(g + x3Cdd)^(-3)*(x3Cddd)^2;	  
x02Cdd = -2*x3Cddd*(g*x2Cd + x03Cd*x2Cdd + x03C*x2Cddd - x2Cddd*x3C - x2Cdd*x3Cd + x2Cd*x3Cdd + x2C*x3Cddd)*(g + x3Cdd)^(-2) -... 
   (g*x2C + x03C*x2Cdd - x2Cdd*x3C + x2C*x3Cdd)*x3Cdddd*(g + x3Cdd)^(-2) +... 
   (g*x2Cdd + x03Cdd*x2Cdd + 2*x03Cd*x2Cddd + x03C*x2Cdddd - x2Cdddd*x3C - 2*x2Cddd*x3Cd + 2*x2Cd*x3Cddd + x2C*x3Cdddd)*...
    (g + x3Cdd)^(-1) + 2*(g*x2C + x03C*x2Cdd - x2Cdd*x3C + x2C*x3Cdd)*(g + x3Cdd)^(-3)*(x3Cddd)^2;

L0C = sqrt((x01C - x41)^2 + (x02C - x42)^2 + (x03C - x43)^2);
L1C = sqrt((x01C - x11)^2 + (x02C - x12)^2 + (x03C - x13)^2);	
L2C = sqrt((x03C - x12)^2 + (x01C - x21)^2 + (x02C - x22)^2);
L3C = sqrt((x01C - x31)^2 + (x02C - x32)^2 + (x03C - x33)^2);	
L4C = sqrt(-2*x01C*x1C - 2*x02C*x2C - 2*x03C*x3C + (x01C)^2 + (x02C)^2 + (x03C)^2 + (x1C)^2 + (x2C)^2 + (x3C)^2) +... 
    sqrt((x01C - x41)^2 + (x02C - x42)^2 + (x03C - x43)^2);


L1Cd =((2*x01Cd*(x01C - x11) + 2*x02Cd*(x02C - x12) + 2*x03Cd*(x03C - x13))*...
      ((x01C - x11)^2 + (x02C - x12)^2 + (x03C - x13)^2)^(-0.5))/2;
L2Cd =((2*x03Cd*(x03C - x12) + 2*x01Cd*(x01C - x21) + 2*x02Cd*(x02C - x22))*...
      ((x03C - x12)^2 + (x01C - x21)^2 + (x02C - x22)^2)^(-0.5))/2;	
L3Cd = ((2*x01Cd*(x01C - x31) + 2*x02Cd*(x02C - x32) + 2*x03Cd*(x03C - x33))*...
      ((x01C - x31)^2 + (x02C - x32)^2 + (x03C - x33)^2)^(-0.5))/2;	
L4Cd = ((2*x01C*x01Cd + 2*x02C*x02Cd + 2*x03C*x03Cd - 2*x01Cd*x1C - 2*x01C*x1Cd + 2*x1C*x1Cd - 2*x02Cd*x2C - 2*x02C*x2Cd +... 
         2*x2C*x2Cd - 2*x03Cd*x3C - 2*x03C*x3Cd + 2*x3C*x3Cd)*...
       (-2*x01C*x1C - 2*x02C*x2C - 2*x03C*x3C + (x01C)^2 + (x02C)^2 + (x03C)^2 + (x1C)^2 + (x2C)^2 + (x3C)^2)^(-0.5))/2. +... 
    ((2*x01Cd*(x01C - x41) + 2*x02Cd*(x02C - x42) + 2*x03Cd*(x03C - x43))*((x01C - x41)^2 + (x02C - x42)^2 + (x03C - x43)^2)^(-0.5))/2;	
	  
L1Cdd = (((x01C - x11)^2 + (x02C - x12)^2 + (x03C - x13)^2)^(-0.5)*...
       (2*x01Cdd*(x01C - x11) + 2*x02Cdd*(x02C - x12) + 2*x03Cdd*(x03C - x13) + 2*(x01Cd)^2 + 2*(x02Cd)^2 + 2*(x03Cd)^2))/2. -... 
    (((x01C - x11)^2 + (x02C - x12)^2 + (x03C - x13)^2)^(-1.5)*(2*x01Cd*(x01C - x11) + 2*x02Cd*(x02C - x12) + 2*x03Cd*(x03C - x13))^2)/4;	  
	  
L2Cdd = (((x03C - x12)^2 + (x01C - x21)^2 + (x02C - x22)^2)^(-0.5)*...
       (2*x03Cdd*(x03C - x12) + 2*x01Cdd*(x01C - x21) + 2*x02Cdd*(x02C - x22) + 2*(x01Cd)^2 + 2*(x02Cd)^2 + 2*(x03Cd)^2))/2. -... 
    (((x03C - x12)^2 + (x01C - x21)^2 + (x02C - x22)^2)^(-1.5)*(2*x03Cd*(x03C - x12) + 2*x01Cd*(x01C - x21) + 2*x02Cd*(x02C - x22))^2)/4;	  

L3Cdd = (((x01C - x31)^2 + (x02C - x32)^2 + (x03C - x33)^2)^(-0.5)*...
       (2*x01Cdd*(x01C - x31) + 2*x02Cdd*(x02C - x32) + 2*x03Cdd*(x03C - x33) + 2*(x01Cd)^2 + 2*(x02Cd)^2 + 2*(x03Cd)^2))/2. - ...
    (((x01C - x31)^2 + (x02C - x32)^2 + (x03C - x33)^2)^(-1.5)*(2*x01Cd*(x01C - x31) + 2*x02Cd*(x02C - x32) + 2*x03Cd*(x03C - x33))^2)/4;

L4Cdd =(((x01C - x41)^2 + (x02C - x42)^2 + (x03C - x43)^2)^(-0.5)*...
       (2*x01Cdd*(x01C - x41) + 2*x02Cdd*(x02C - x42) + 2*x03Cdd*(x03C - x43) + 2*(x01Cd)^2 + 2*(x02Cd)^2 + 2*(x03Cd)^2))/2. +... 
    ((-2*x01C*x1C - 2*x02C*x2C - 2*x03C*x3C + (x01C)^2 + (x02C)^2 + (x03C)^2 + (x1C)^2 + (x2C)^2 + (x3C)^2)^(-0.5)*...
       (2*x01C*x01Cdd + 2*x02C*x02Cdd + 2*x03C*x03Cdd - 2*x01Cdd*x1C - 4*x01Cd*x1Cd - 2*x01C*x1Cdd + 2*x1C*x1Cdd - 2*x02Cdd*x2C -... 
         4*x02Cd*x2Cd - 2*x02C*x2Cdd + 2*x2C*x2Cdd - 2*x03Cdd*x3C - 4*x03Cd*x3Cd - 2*x03C*x3Cdd + 2*x3C*x3Cdd + 2*(x01Cd)^2 + ...
         2*(x02Cd)^2 + 2*(x03Cd)^2 + 2*(x1Cd)^2 + 2*(x2Cd)^2 + 2*(x3Cd)^2))/2. - ...
    ((-2*x01C*x1C - 2*x02C*x2C - 2*x03C*x3C + (x01C)^2 + (x02C)^2 + (x03C)^2 + (x1C)^2 + (x2C)^2 + (x3C)^2)^(-1.5)*...
       (2*x01C*x01Cd + 2*x02C*x02Cd + 2*x03C*x03Cd - 2*x01Cd*x1C - 2*x01C*x1Cd + 2*x1C*x1Cd - 2*x02Cd*x2C - 2*x02C*x2Cd + 2*x2C*x2Cd - ...
         2*x03Cd*x3C - 2*x03C*x3Cd + 2*x3C*x3Cd)^2)/4. - ...
    (((x01C - x41)^2 + (x02C - x42)^2 + (x03C - x43)^2)^(-1.5)*(2*x01Cd*(x01C - x41) + 2*x02Cd*(x02C - x42) + 2*x03Cd*(x03C - x43))^2)/4;


l1C = -(m*(g + x3Cdd)*(x03C - x3C)^(-1));	
		  

l2C = (L4C*(-(x03C*x22*x31) + x03C*x21*x32 + x03C*x22*x41 - x03C*x32*x41 + x12*x32*x41 - x22*x33*x41 - x03C*x21*x42 + x03C*x31*x42 -... 
        x12*x31*x42 + x21*x33*x42 + x22*x31*x43 - x21*x32*x43 + x02C*(x12*x31 - x21*x33 - x12*x41 + x33*x41 + x21*x43 - x31*x43) + ...
        x01C*(x22*x33 - x33*x42 + x12*(-x32 + x42) - x22*x43 + x32*x43))*l1C +... 
     L0C*(-(mo*x02Cdd*x03C*x21) + mo*x01Cdd*x03C*x22 + mo*x02Cdd*x03C*x31 - mo*x02Cdd*x12*x31 + g*mo*x22*x31 + mo*x03Cdd*x22*x31 -... 
        mo*x01Cdd*x03C*x32 + mo*x01Cdd*x12*x32 - g*mo*x21*x32 - mo*x03Cdd*x21*x32 + mo*x02Cdd*x21*x33 - mo*x01Cdd*x22*x33 + ...
        x03C*x1C*x22*l1C - x03C*x21*x2C*l1C + x03C*x2C*x31*l1C - x12*x2C*x31*l1C - x03C*x1C*x32*l1C + x12*x1C*x32*l1C - ...
        x1C*x22*x33*l1C + x21*x2C*x33*l1C + x22*x31*x3C*l1C - x21*x32*x3C*l1C - x03C*x22*x41*l1C + x03C*x32*x41*l1C - ...
        x12*x32*x41*l1C + x22*x33*x41*l1C + x03C*x21*x42*l1C - x03C*x31*x42*l1C + x12*x31*x42*l1C - x21*x33*x42*l1C - ...
        x22*x31*x43*l1C + x21*x32*x43*l1C + x02C*(mo*((g + x03Cdd)*(x21 - x31) + x01Cdd*(-x12 + x33)) + ...
           (x1C*(-x12 + x33) + (x21 - x31)*x3C + x12*x41 - x33*x41 - x21*x43 + x31*x43)*l1C) + ...
        x01C*(mo*(-((g + x03Cdd)*(x22 - x32)) + x02Cdd*(x12 - x33)) + ...
      (x2C*(x12 - x33) + (-x22 + x32)*x3C - x12*x42 + x33*x42 + x22*x43 - x32*x43)*l1C)))*(L0C)^(-1)*...
     (x03C*x12*x21 - x03C*x11*x22 - x03C*x12*x31 + x03C*x22*x31 - x13*x22*x31 + x03C*x11*x32 - x11*x12*x32 - x03C*x21*x32 +... 
     x13*x21*x32 - x12*x21*x33 + x11*x22*x33 + x02C*(x11*x12 - x13*x21 - x12*x31 + x13*x31 - x11*x33 + x21*x33) + ...
     x01C*(x13*x22 - x13*x32 - x22*x33 + x12*(x32 + x33) - (x12)^2) + x31*(x12)^2)^(-1);

l3C=(L4C*(-(x03C*x12*x31) + x03C*x11*x32 + x03C*x12*x41 - x03C*x32*x41 + x13*x32*x41 - x12*x33*x41 - x03C*x11*x42 + x03C*x31*x42 -... 
     x13*x31*x42 + x11*x33*x42 + x12*x31*x43 - x11*x32*x43 + x02C*(x13*x31 - x11*x33 - x13*x41 + x33*x41 + x11*x43 - x31*x43) + ...
     x01C*(x12*x33 - x33*x42 + x13*(-x32 + x42) - x12*x43 + x32*x43))*l1C +... 
     L0C*(-(mo*x02Cdd*x03C*x11) + mo*x01Cdd*x03C*x12 + mo*x02Cdd*x03C*x31 + g*mo*x12*x31 + mo*x03Cdd*x12*x31 - mo*x02Cdd*x13*x31 -... 
     mo*x01Cdd*x03C*x32 - g*mo*x11*x32 - mo*x03Cdd*x11*x32 + mo*x01Cdd*x13*x32 + mo*x02Cdd*x11*x33 - mo*x01Cdd*x12*x33 + ...
     x03C*x12*x1C*l1C - x03C*x11*x2C*l1C + x03C*x2C*x31*l1C - x13*x2C*x31*l1C - x03C*x1C*x32*l1C + x13*x1C*x32*l1C - ...
     x12*x1C*x33*l1C + x11*x2C*x33*l1C + x12*x31*x3C*l1C - x11*x32*x3C*l1C - x03C*x12*x41*l1C + x03C*x32*x41*l1C - ...
     x13*x32*x41*l1C + x12*x33*x41*l1C + x03C*x11*x42*l1C - x03C*x31*x42*l1C + x13*x31*x42*l1C - x11*x33*x42*l1C - ...
     x12*x31*x43*l1C + x11*x32*x43*l1C + x02C*(mo*((g + x03Cdd)*(x11 - x31) + x01Cdd*(-x13 + x33)) + ...
     (x1C*(-x13 + x33) + (x11 - x31)*x3C + x13*x41 - x33*x41 - x11*x43 + x31*x43)*l1C) + ...
     x01C*(mo*(-((g + x03Cdd)*(x12 - x32)) + x02Cdd*(x13 - x33)) + ...
     (x2C*(x13 - x33) + (-x12 + x32)*x3C - x13*x42 + x33*x42 + x12*x43 - x32*x43)*l1C)))*(L0C)^(-1)*...
     (-(x03C*x12*x21) + x03C*x11*x22 + x03C*x12*x31 - x03C*x22*x31 + x13*x22*x31 - x03C*x11*x32 + x11*x12*x32 + x03C*x21*x32 -... 
     x13*x21*x32 + x12*x21*x33 - x11*x22*x33 + x02C*(-(x11*x12) + x13*x21 + x12*x31 - x13*x31 + x11*x33 - x21*x33) - x31*(x12)^2 +... 
     x01C*(-(x13*x22) + x13*x32 + x22*x33 - x12*(x32 + x33) + (x12)^2))^(-1);	
l5C = -((L0C - L4C)*l1C*L0C^(-1));
l4C = (g*mo + mo*x03Cdd - (x03C - x3C)*l1C - (x03C - x13)*l2C - (x03C - x12)*l3C - (x03C - x43)*l5C)*(x03C - x33)^(-1);


Err(1) = (L1-L1C);
Err(2) = (L2-L2C);
Err(3) = (L3-L3C);
Err(4) = (L4-L4C);

Errd(1) = (L1d-L1Cd);
Errd(2) = (L2d-L2Cd);
Errd(3) = (L3d-L3Cd);
Errd(4) = (L4d-L4Cd);
 
%Calcul des entres de consigne(a priori)
T1C = (L1C*l2C) + (m1*L1Cdd);
T2C = (L2C*l3C) + (m2*L2Cdd);
T3C = (L3C*l4C) + (m3*L3Cdd);
T4C = -(L0C-L4C)*l1C + (m4*L4Cdd);



%Calcul des entres (rgulateurs + commande a priori)
T1 = T1C-KD*Errd(1)-KP*Err(1);
T2 = T2C-KD*Errd(2)-KP*Err(2);
T3 = T3C-KD*Errd(3)-KP*Err(3);
T4 = T4C-KD*Errd(4)-KP*Err(4);

T = [T1 T2 T3 T4];

sys(1)  = T1;
sys(2)  = T2;
sys(3)  = T3;
sys(4)  = T4;

sys(5)  = T1C;
sys(6)  = T2C;
sys(7)  = T3C;
sys(8)  = T4C;

sys(9)  = Err(1);
sys(10)  = Err(2);
sys(11)  = Err(3);
sys(12)  = Err(4);

sys(13)  = Errd(1);
sys(14)  = Errd(2);
sys(15)  = Errd(3);
sys(16)  = Errd(4);


% end mdlOutputs


